@@ -41,26 +41,43 @@ gchar *temp_bg = NULL;
 int current_item = 0;
 int current_list = 0;
 GHashTable *config = NULL;
-#define SERVER_PORT 2346
+GHashTable *miniviews = NULL;
 
 // Command line options
 gboolean windowed = FALSE;
 gboolean debugging = FALSE;
-int server_port = SERVER_PORT;
+int server_port = 2346;
+gchar *server_ip = "";
+gchar *server_type = "normal";
+int server_mode = NORMAL_SERVER;
 gchar *dbhostname = "localhost";
 gchar *geometry = NULL;
+gchar *profile = NULL;
+gchar *extra_data = NULL;
 unsigned long windowid = 0;
+gchar hostname[32];
+gchar ipaddr[16] = "127.0.0.1\0";
+guint tracker_timeout = 0;
+
 
 static GOptionEntry entries[] = {
-    {"window", 'w', 0, G_OPTION_ARG_NONE, &windowed, "Run in a window", NULL},
+    {"type", 't', 0, G_OPTION_ARG_STRING, &server_type, "Server type",
+     "[normal | preview | miniview | simple | headless ] "},
+    {"profile", 'p', 0, G_OPTION_ARG_STRING, &profile, "Profile",
+     "profile_name"},
     {"remote", 'r', 0, G_OPTION_ARG_STRING, &dbhostname, "Database hostname",
-     NULL},
+     "hostname"},
     {"geometry", 'g', 0, G_OPTION_ARG_STRING, &geometry, "Window Geometry",
-     NULL},
-    {"port", 'p', 0, G_OPTION_ARG_INT, &server_port, "Port to listen on",
-     NULL},
+     "geom"},
+    {"listen", 'l', 0, G_OPTION_ARG_INT, &server_port, "Port to listen on",
+     "port_number"},
+    {"ipaddr", 'i', 0, G_OPTION_ARG_STRING, &server_ip, "IP to listen on",
+     "ip_number"},
     {"miniview", 'm', 0, G_OPTION_ARG_INT, &windowid, "Embed in windowid",
-     NULL},
+     "windowid"},
+    {"extra", 'e', 0, G_OPTION_ARG_STRING, &extra_data, "Extra data to report to avahi",
+     "data"},
+    {"window", 'w', 0, G_OPTION_ARG_NONE, &windowed, "Run in a window", NULL},
     {"debug", 'd', 0, G_OPTION_ARG_NONE, &debugging, "Enable debug output",
      NULL},
     {NULL}
@@ -90,7 +107,74 @@ main (int argc, char *argv[])
     }
     int ret = db_select ();
     if (ret) {
-        // Really should handle this ;)
+        // Always true
+    }
+    gethostname(hostname,sizeof(hostname));
+
+    if (g_strcmp0(server_ip,"") != 0 ) {
+        g_snprintf(ipaddr,sizeof(ipaddr),"%s",server_ip);
+    } else {
+    
+        // Find the external IP address (by finding default route)
+        FILE *f;
+        char line[100] , *p , *c;
+
+        f = fopen("/proc/net/route" , "r");
+        while(fgets(line , 100 , f)) {
+            p = strtok(line , " \t");
+            c = strtok(NULL , " \t");
+
+            if(p!=NULL && c!=NULL) {
+                if(strcmp(c , "00000000") == 0) {
+                    //printf("Default interface is : %s \n" , p);
+                    break;
+                }
+            }
+        }
+
+        int fm = AF_INET;
+        struct ifaddrs *ifaddr, *ifa;
+        int family , s;
+
+        if (getifaddrs(&ifaddr) == -1) {
+            perror("getifaddrs");
+            exit(EXIT_FAILURE);
+        }
+        for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+            if (ifa->ifa_addr == NULL) {
+                continue;
+            }
+    
+            family = ifa->ifa_addr->sa_family;
+    
+            if(strcmp( ifa->ifa_name , p) == 0) {
+                if (family == fm) {
+                    s = getnameinfo( ifa->ifa_addr, (family == AF_INET) ? sizeof(struct sockaddr_in) : sizeof(struct sockaddr_in6) , ipaddr, sizeof(ipaddr), NULL , 0 , NI_NUMERICHOST);
+    
+                    if (s != 0) {
+                        printf("getnameinfo() failed: %s\n", gai_strerror(s));
+                        exit(EXIT_FAILURE);
+                    }
+                }
+            }
+        }
+        freeifaddrs(ifaddr);
+    }
+
+    do_query(FALSE, lyricDb, "SELECT profile FROM profiles WHERE host='%s'", hostname);
+    MYSQL_ROW row;
+    MYSQL_RES *result;
+    result = mysql_store_result (lyricDb);
+    row = mysql_fetch_row (result);
+    if (row != NULL) {
+        if (profile == NULL) {
+            profile=g_strdup(row[0]);
+        }
+    } else {
+        if (profile == NULL) {
+            profile = g_strdup("Default");
+        }
+        do_query(FALSE, lyricDb, "INSERT INTO profiles (host, profile) VALUES ('%s', '%s')", hostname, profile);
     }
     load_configuration (lyricDb);
     bible_load ((gchar *) g_hash_table_lookup (config, "DefBible"));
@@ -99,29 +183,57 @@ main (int argc, char *argv[])
     // Setup network
     GSocketService *service = g_socket_service_new ();
     GInetAddress *address = g_inet_address_new_any (G_SOCKET_FAMILY_IPV4);
-    g_snprintf (argv[0], 29, "Lyricue Display on port %04d", server_port);
-    l_debug ("Process Name:%s", argv[0]);
 
     if (!g_socket_listener_add_inet_port
         (G_SOCKET_LISTENER (service), server_port, NULL, NULL)) {
         l_debug ("Unable to listen on port %d", server_port);
-        return EXIT_FAILURE;
+        server_port = g_socket_listener_add_any_inet_port
+                        (G_SOCKET_LISTENER (service), NULL, NULL);
+        if (server_port == 0) {
+            return EXIT_FAILURE;
+        }
     }
+    l_debug("Listening on port %d", server_port);
     g_object_unref (address);
     g_socket_service_start (service);
     g_signal_connect (service, "incoming", G_CALLBACK (new_connection), NULL);
 
+    if (g_strcmp0(server_type, "headless") != 0) {
+        ret = create_main_window (argc, argv);
+    }
+
+    // Publish to avahi (zeroconf/bonjour)
+    publish_avahi(server_port, server_type);
+    if (g_strcmp0(server_type, "normal") == 0) {
+        server_mode=NORMAL_SERVER;
+    } else if (g_strcmp0(server_type, "preview") == 0) {
+        server_mode=PREVIEW_SERVER;
+    } else if (g_strcmp0(server_type, "miniview") == 0) {
+        server_mode=MINIVIEW_SERVER;
+    } else if (g_strcmp0(server_type, "simple") == 0) {
+        server_mode=SIMPLE_SERVER;
+    } else if (g_strcmp0(server_type, "headless") == 0) {
+        server_mode=HEADLESS_SERVER;
+    }
+
+    // Create tracker update timeout
+    tracker_timeout = g_timeout_add_seconds (1, (GSourceFunc) update_tracker,
+                                         NULL);
+
+
     // Setup tracker entry in DB
-    do_query (lyricDb, "DELETE FROM playlists WHERE id=-1");
-    do_query (lyricDb, "INSERT INTO playlists SET id=-1,ref=0,title=''");
-
-    ret = create_main_window (argc, argv);
-
-    if (windowid == 0) {
+    do_query (FALSE, lyricDb, "DELETE FROM status WHERE host='%s:%d'",hostname, server_port);
+    do_query (FALSE, lyricDb, "INSERT INTO status SET host='%s:%d',ref=0,title='', profile='%s', type='%s%s', ip='%s'",hostname, server_port, profile, server_type, extra_data, ipaddr);
+    if (server_mode==NORMAL_SERVER) {
         clutter_main ();
     } else {
         gtk_main ();
     }
+
+    do_query (FALSE, lyricDb,
+              "UPDATE status SET lastupdate = 0 WHERE host=\"%s:%d\"",
+              hostname, server_port);
+    unpublish_avahi();
     ret = db_deselect ();
 
     l_debug("Exiting");
@@ -212,7 +324,7 @@ handle_command (GIOChannel * source, const char *command)
         } else if (g_strcmp0 (line[0], "next_point") == 0) {
             do_next_point (line[1]);
         } else if (g_strcmp0 (line[0], "get") == 0) {
-            do_get (line[1]);
+            returnstring = do_get (line[1]);
         } else if (g_strcmp0 (line[0], "display") == 0) {
             do_display (line[1],FALSE);
         } else if (g_strcmp0 (line[0], "osd") == 0) {
@@ -229,19 +341,24 @@ handle_command (GIOChannel * source, const char *command)
             returnstring = do_query_json (line[1]);
         } else if (g_strcmp0 (line[0], "bible") == 0) {
             returnstring = do_bible(line[1]);
+        } else if (g_strcmp0 (line[0], "profile") == 0) {
+            do_profile_change(line[1]);
         }
     }
     g_strfreev (line);
     if (returnstring != NULL) {
-        GRegex * re = g_regex_new("\"(HEX\(.*\))\":\"[0-9A-F]+\"",0,0,NULL);
-        gchar * status  = g_regex_replace(re, returnstring->str, returnstring->len, 0, "\\1:(Hex_encoded_data)" ,G_REGEX_MATCH_NOTEMPTY,NULL);
-        g_regex_unref(re);
-        l_debug ("The status message sent is (%d chars): %s", strlen(status), status);
-        g_free(status);
+        if ((returnstring->len > 1024) && (g_strrstr_len(returnstring->str, returnstring->len, " ") == NULL)){
+            l_debug ("The status message sent is (%d chars): [uuencoded data]", returnstring->len);
+        } else {
+            l_debug ("The status message sent is (%d chars): %s", returnstring->len, returnstring->str);
+        }
         gsize bytes_written = 0;
         GError *err = NULL;
         GIOStatus res = g_io_channel_write_chars (source, returnstring->str,
                                                   returnstring->len, &bytes_written, &err);
+        if (res) {
+            // Ignore it
+        }
         gsize total = bytes_written;
         // Repeat until all sent - needed since socket is non-blocking
         while (bytes_written < returnstring->len) {
@@ -385,11 +502,15 @@ void
 do_backdrop (const char *options)
 {
     l_debug ("do_backdrop: %s", options);
-    gchar **line = g_strsplit (options, ":", 2);
-    temp_bg = NULL;
-    default_bg = parse_special (line[0]);
-    change_backdrop (default_bg, TRUE, DEFAULT);
-    g_strfreev (line);
+    if (server_mode != SIMPLE_SERVER) {
+        gchar **line = g_strsplit (options, ":", 2);
+        temp_bg = NULL;
+        default_bg = parse_special (line[0]);
+        change_backdrop (default_bg, TRUE, DEFAULT);
+        g_strfreev (line);
+    } else {
+        if (background != NULL) destroy_actor(background);
+    }
 }
 
 void
@@ -452,12 +573,6 @@ do_next_point (const char *options)
     l_debug ("do_next_point not implemented");
 }
 
-void
-do_get (const char *options)
-{
-    l_debug ("do_get not implemented");
-}
-
 void
 do_osd (const char *options)
 {
@@ -489,7 +604,7 @@ do_display (const char *options, const int quick_show)
         MYSQL_ROW row;
         MYSQL_RES *result;
         gboolean bg_changed = FALSE;
-        do_query (lyricDb, "SELECT playlist FROM playlist WHERE playorder=%d",
+        do_query (FALSE, lyricDb, "SELECT playlist FROM playlist WHERE playorder=%d",
                   current_item);
         result = mysql_store_result (lyricDb);
         row = mysql_fetch_row (result);
@@ -512,7 +627,7 @@ do_display (const char *options, const int quick_show)
                 bg_changed = TRUE;
             }
         } else if (g_strcmp0 (command, "next_page") == 0) {
-            do_query (lyricDb,
+            do_query (FALSE, lyricDb,
                       "SELECT MIN(playorder) FROM playlist WHERE playlist=%d AND playorder > %d ORDER BY playorder",
                       current_list, current_item);
             result = mysql_store_result (lyricDb);
@@ -532,7 +647,7 @@ do_display (const char *options, const int quick_show)
                     }
                     if (loop_parent == 0) {
                         l_debug ("Looping a song, back to page 1");
-                        do_query (lyricDb,
+                        do_query (FALSE, lyricDb,
                                   "SELECT MIN(playorder) FROM playlist WHERE playlist=%d",
                                   current_list);
                         result = mysql_store_result (lyricDb);
@@ -543,7 +658,7 @@ do_display (const char *options, const int quick_show)
                         }
                     } else {
                         l_debug ("Looping a sublist");
-                        do_query (lyricDb,
+                        do_query (FALSE, lyricDb,
                                   "SELECT MIN(p1.playorder) FROM playlist AS p1, playlist AS p2 WHERE p1.playorder>p2.playorder AND p2.type='play' AND p2.data=%d AND p1.playlist=%d",
                                   current_list, loop_parent);
                         result = mysql_store_result (lyricDb);
@@ -553,7 +668,7 @@ do_display (const char *options, const int quick_show)
                             current_item = atoi (row[0]);
                         } else {
                             // Loop back to top of parent
-                            do_query (lyricDb,
+                            do_query (FALSE, lyricDb,
                                       "SELECT MIN(playorder) FROM playlist WHERE playlist=%d",
                                       loop_parent);
                             result = mysql_store_result (lyricDb);
@@ -572,7 +687,7 @@ do_display (const char *options, const int quick_show)
             }
 
         } else if (g_strcmp0 (command, "prev_page") == 0) {
-            do_query (lyricDb,
+            do_query (FALSE, lyricDb,
                       "SELECT MAX(playorder) FROM playlist WHERE playlist=%d AND playorder < %d ORDER BY playorder",
                       current_list, current_item);
             result = mysql_store_result (lyricDb);
@@ -583,7 +698,7 @@ do_display (const char *options, const int quick_show)
             } else {
                 if (g_strcmp0 (line[1], "loop") == 0) {
                     // Loop back to end of playlist
-                    do_query (lyricDb,
+                    do_query (FALSE, lyricDb,
                               "SELECT MAX(playorder) FROM playlist WHERE playlist=%d",
                               current_list);
                     result = mysql_store_result (lyricDb);
@@ -594,7 +709,7 @@ do_display (const char *options, const int quick_show)
                     }
                 } else {
                     // Jump back to last page of previous song
-                    do_query(lyricDb,
+                    do_query(FALSE, lyricDb,
                              "SELECT MAX(playorder) FROM playlist "
                              "WHERE playlist="
                              "(SELECT data FROM playlist "
@@ -618,7 +733,7 @@ do_display (const char *options, const int quick_show)
             }
 
         } else if (g_strcmp0 (command, "next_song") == 0) {
-            do_query (lyricDb,
+            do_query (FALSE, lyricDb,
                       "SELECT a.playorder,a.playlist FROM playlist AS a, playlist AS b WHERE a.data=b.playlist AND a.type=\"play\" AND b.playorder=%d",
                       current_item);
             result = mysql_store_result (lyricDb);
@@ -629,7 +744,7 @@ do_display (const char *options, const int quick_show)
                 current_item = atoi (row[0]);
                 current_list = atoi (row[1]);
             }
-            do_query (lyricDb,
+            do_query (FALSE, lyricDb,
                       "SELECT MIN(playorder) FROM playlist WHERE playorder > %d AND playlist=%d",
                       current_item, current_list);
             result = mysql_store_result (lyricDb);
@@ -640,7 +755,7 @@ do_display (const char *options, const int quick_show)
             mysql_free_result (result);
 
         } else if (g_strcmp0 (command, "prev_song") == 0) {
-            do_query (lyricDb,
+            do_query (FALSE, lyricDb,
                       "SELECT a.playorder,a.playlist FROM playlist AS a, playlist AS b WHERE a.data=b.playlist AND a.type=\"play\" AND b.playorder=%d",
                       current_item);
             result = mysql_store_result (lyricDb);
@@ -650,7 +765,7 @@ do_display (const char *options, const int quick_show)
                 current_item = atoi (row[0]);
                 current_list = atoi (row[1]);
             }
-            do_query (lyricDb,
+            do_query (FALSE, lyricDb,
                       "SELECT MAX(playorder) FROM playlist WHERE playorder < %d AND playlist=%d",
                       current_item, current_list);
             result = mysql_store_result (lyricDb);
@@ -661,7 +776,7 @@ do_display (const char *options, const int quick_show)
             }
 
         } else if (g_strcmp0 (command, "page") == 0) {
-            do_query (lyricDb,
+            do_query (FALSE, lyricDb,
                       "SELECT playorder FROM playlist WHERE playlist=%d",
                       current_list);
             result = mysql_store_result (lyricDb);
@@ -677,7 +792,7 @@ do_display (const char *options, const int quick_show)
             current_item = atoi (command);
         }
 
-        do_query (lyricDb,
+        do_query (FALSE, lyricDb,
                   "SELECT type,data,transition FROM playlist WHERE playorder=%d",
                   current_item);
         result = mysql_store_result (lyricDb);
@@ -696,8 +811,10 @@ do_display (const char *options, const int quick_show)
 
             if (g_strcmp0 (type, "back") == 0) {
                 default_bg = g_strdup (data);
-                change_backdrop (default_bg, TRUE, transition);
-                bg_changed = TRUE;
+                if (server_mode == NORMAL_SERVER) {
+                    change_backdrop (default_bg, TRUE, transition);
+                    bg_changed = TRUE;
+                }
                 g_strfreev (line);
             } else if (g_strcmp0 (type, "file") == 0) {
                 change_backdrop (data, FALSE, transition);
@@ -707,7 +824,7 @@ do_display (const char *options, const int quick_show)
                 change_backdrop (data, FALSE, transition);
                 bg_changed = TRUE;
             } else if (g_strcmp0 (type, "vers") == 0) {
-                do_query (lyricDb,
+                do_query (FALSE, lyricDb,
                           "SELECT title FROM playlist,playlists WHERE playlist.playlist=playlists.id AND playorder=%d",
                           current_item);
                 result = mysql_store_result (lyricDb);
@@ -734,7 +851,7 @@ do_display (const char *options, const int quick_show)
                 wrap = TRUE;
             } else if ((g_strcmp0 (type, "play") == 0) ||
                        (g_strcmp0 (type, "sub") == 0)) {
-                do_query (lyricDb,
+                do_query (FALSE, lyricDb,
                           "SELECT playorder FROM playlist WHERE playlist=%s ORDER BY playorder",
                           data);
                 result = mysql_store_result (lyricDb);
@@ -742,7 +859,7 @@ do_display (const char *options, const int quick_show)
                 do_display (row[0],FALSE);
                 return;
             } else {            // Song page
-                do_query (lyricDb,
+                do_query (FALSE, lyricDb,
                           "SELECT title,artist,lyrics,copyright FROM lyricMain AS l, page AS pa WHERE (pa.songid=l.id OR pa.songid=-l.id) AND pa.pageid=%s",
                           data);
                 result = mysql_store_result (lyricDb);
@@ -772,37 +889,41 @@ do_display (const char *options, const int quick_show)
 
             // Look for associated background image
             if (!bg_changed) {
-                int res = do_query (lyricDb,
-                                    "SELECT imagename FROM associations WHERE playlist=%d",
-                                    current_item);
-                int bg_changed = FALSE;
-                if (res == 0) {
-                    result = mysql_store_result (lyricDb);
-                    row = mysql_fetch_row (result);
-                    if (row != NULL) {
-                        change_backdrop (row[0], TRUE, transition);
-                        bg_changed = TRUE;
-                    }
-                    mysql_free_result (result);
-                }
-                if (!bg_changed) {
-                    res =
-                      do_query (lyricDb,
-                                "SELECT a.imagename,q.data FROM associations as a, playlist AS p, playlist AS q WHERE p.type='play' AND p.data=q.playlist and a.playlist=p.playorder AND q.playorder=%d",
-                                current_item);
+                if (server_mode == SIMPLE_SERVER) {
+                    if (background != NULL) clutter_actor_destroy(background);
+                } else {
+                    int res = do_query (FALSE, lyricDb,
+                                        "SELECT imagename FROM associations WHERE playlist=%d",
+                                        current_item);
+                    int bg_changed = FALSE;
                     if (res == 0) {
                         result = mysql_store_result (lyricDb);
                         row = mysql_fetch_row (result);
-                        mysql_free_result (result);
                         if (row != NULL) {
                             change_backdrop (row[0], TRUE, transition);
                             bg_changed = TRUE;
                         }
+                        mysql_free_result (result);
+                    }
+                    if (!bg_changed) {
+                        res =
+                          do_query (FALSE, lyricDb,
+                                    "SELECT a.imagename,q.data FROM associations as a, playlist AS p, playlist AS q WHERE p.type='play' AND p.data=q.playlist and a.playlist=p.playorder AND q.playorder=%d",
+                                    current_item);
+                        if (res == 0) {
+                            result = mysql_store_result (lyricDb);
+                            row = mysql_fetch_row (result);
+                            mysql_free_result (result);
+                            if (row != NULL) {
+                                change_backdrop (row[0], TRUE, transition);
+                                bg_changed = TRUE;
+                            }
+                        }
+                    }
+                    if (!bg_changed && (g_strcmp0 (default_bg, current_bg) != 0)) {
+                        l_debug ("Reset bg to default");
+                        change_backdrop (default_bg, TRUE, transition);
                     }
-                }
-                if (!bg_changed && (g_strcmp0 (default_bg, current_bg) != 0)) {
-                    l_debug ("Reset bg to default");
-                    change_backdrop (default_bg, TRUE, transition);
                 }
             }
 
@@ -821,52 +942,55 @@ update_tracker ()
 {
     //l_debug ("Updating tracker");
 
-    // Only do if this is main server
-    if (server_port == SERVER_PORT) {
-        GString *title = g_string_new (NULL);
-        if (blanked_state == BLANK_BG) {
-            g_string_assign (title, "blank_bg");
-        } else if (blanked_state == BLANK_TEXT) {
-            g_string_assign (title, "blank_text");
-        }
-        if (bg_is_video) {
-            g_string_append_printf (title, "%.0f;%.0f;%d",
-                                    clutter_media_get_progress (CLUTTER_MEDIA
-                                                                (background))
-                                    *
-                                    clutter_media_get_duration (CLUTTER_MEDIA
-                                                                (background)),
-                                    clutter_media_get_duration (CLUTTER_MEDIA
-                                                                (background)),
-                                    clutter_media_get_playing (CLUTTER_MEDIA
-                                                               (background)));
-        } else {
-            g_string_append (title, "0;0;0");
-        }
-        do_query (lyricDb,
-                  "UPDATE playlists SET ref = %d, title = \"%s\" WHERE id=-1",
-                  current_item, g_string_free (title, FALSE));
+    GString *title = g_string_new (NULL);
+    if (blanked_state == BLANK_BG) {
+        g_string_assign (title, "blank_bg");
+    } else if (blanked_state == BLANK_TEXT) {
+        g_string_assign (title, "blank_text");
     }
+    if (bg_is_video) {
+        g_string_append_printf (title, "%.0f;%.0f;%d",
+                                clutter_media_get_progress (CLUTTER_MEDIA
+                                                            (background))
+                                *
+                                clutter_media_get_duration (CLUTTER_MEDIA
+                                                            (background)),
+                                clutter_media_get_duration (CLUTTER_MEDIA
+                                                            (background)),
+                                clutter_media_get_playing (CLUTTER_MEDIA
+                                                           (background)));
+    } else {
+        g_string_append (title, "0;0;0");
+    }
+    do_query (TRUE, lyricDb,
+              "UPDATE status SET ref = %d, title = \"%s\", lastupdate = NOW() WHERE host=\"%s:%d\"",
+              current_item, g_string_free (title, FALSE), hostname, server_port);
     return TRUE;
 }
 
 void
 update_miniview (const char *command)
 {
-    if (server_port == SERVER_PORT) {
+    if (server_mode == NORMAL_SERVER || server_mode==HEADLESS_SERVER) {
         l_debug ("miniview time");
-        GSocketClient *client = g_socket_client_new ();
-        GSocketConnection *conn =
-          g_socket_client_connect_to_host (client, dbhostname, 2348, NULL,
-                                           NULL);
-        if (conn != NULL) {
-            GOutputStream *out =
-              g_io_stream_get_output_stream (G_IO_STREAM (conn));
-            g_output_stream_write (out, command, strlen (command), NULL,
-                                   NULL);
-            g_object_unref (conn);
+        GHashTableIter iter;
+        gpointer key, value;
+        g_hash_table_iter_init(&iter, miniviews);
+        while (g_hash_table_iter_next(&iter, &key, &value)) {
+            l_debug("Updating miniview %s on %s",key,value);
+            GSocketClient *client = g_socket_client_new ();
+            GSocketConnection *conn =
+              g_socket_client_connect_to_host (client, value, 2348, NULL,
+                                               NULL);
+            if (conn != NULL) {
+                GOutputStream *out =
+                  g_io_stream_get_output_stream (G_IO_STREAM (conn));
+                g_output_stream_write (out, command, strlen (command), NULL,
+                                       NULL);
+                g_object_unref (conn);
+            }
+            g_object_unref (client);
         }
-        g_object_unref (client);
     }
 }
 
@@ -905,13 +1029,13 @@ do_query_json (const char *options)
         MYSQL_RES *result;
 
         if (g_strcmp0(line[0],"lyricdb") == 0) {
-            do_query (lyricDb,"%s",parse_special(line[1]));
+            do_query (FALSE, lyricDb,"%s",parse_special(line[1]));
             result = mysql_store_result (lyricDb);
         } else if (g_strcmp0(line[0],"mediadb") == 0) {
-            do_query (mediaDb,"%s",parse_special(line[1]));
+            do_query (FALSE, mediaDb,"%s",parse_special(line[1]));
             result = mysql_store_result (mediaDb);
         } else if (g_strcmp0(line[0],"bibledb") == 0) {
-            do_query (bibleDb,"%s",parse_special(line[1]));
+            do_query (FALSE, bibleDb,"%s",parse_special(line[1]));
             result = mysql_store_result (bibleDb);
         } else {
             g_strfreev (line);
@@ -957,3 +1081,16 @@ do_query_json (const char *options)
     g_strfreev (line);
     return NULL;
 }
+
+void
+do_profile_change(const char *options)
+{
+    gchar **line = g_strsplit (options, ":", 2);
+    if (line[0] != NULL) {
+        profile = g_strdup(line[0]);
+        load_configuration();
+        update_service_profile();
+    }
+    
+    g_strfreev (line);
+}
